@@ -1,5 +1,4 @@
 # Copyright Kevin Deldycke <kevin@deldycke.com> and contributors.
-# All Rights Reserved.
 #
 # This program is Free Software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -19,22 +18,18 @@
 
 from __future__ import annotations
 
-import inspect
 import os
 import re
-import sys
 from configparser import RawConfigParser
-from functools import partial
 from gettext import gettext as _
 from operator import getitem
-from types import ModuleType
-from typing import Iterable, NamedTuple
+from typing import NamedTuple
 
 import regex as re3
 from boltons.strutils import complement_int_list, int_ranges_from_int_list
 from click import Parameter, echo, get_current_context
 from click.core import ParameterSource
-from cloup import Choice, Context, HelpFormatter, HelpTheme, Style, option
+from cloup import Choice, Context, HelpFormatter, Style
 from cloup._util import identity
 from cloup.styling import IStyle
 
@@ -45,7 +40,8 @@ class HelpExtraTheme(NamedTuple):
     """Extends ``cloup.HelpTheme`` with Click Extra's specific properties and
     ``logging.levels``.
 
-    We had to redefined all fields and couldn't extend ``cloup.HelpTheme`` as there is no way to cleanly do it because of mypy. See:
+    We had to redefined all fields and couldn't extend ``cloup.HelpTheme`` as there is
+    no way to cleanly do it because of mypy. See:
     https://github.com/python/typing/issues/427
     https://mypy.readthedocs.io/en/stable/runtime_troubles.html#future-annotations-import-pep-563
     """
@@ -171,9 +167,11 @@ color_env_vars = {
     "NO_COLOR": False,
     "NO_COLORS": False,
 }
-"""List of environment variables recognized as flags to switch color rendering on or off.
+"""List of environment variables recognized as flags to switch color rendering on or
+off.
 
-The key is the name of the variable and the boolean value the value to pass to ``--color`` option flag when encountered.
+The key is the name of the variable and the boolean value the value to pass to
+``--color`` option flag when encountered.
 
 Source: https://github.com/pallets/click/issues/558
 """
@@ -199,12 +197,12 @@ class ColorOption(ExtraOption):
         colorize_from_env = set()
         for var, default in color_env_vars.items():
             if var in os.environ:
-                # Presence of the variable in the environment without a value encodes for an activation,
-                # hence the default to True.
+                # Presence of the variable in the environment without a value encodes
+                # for an activation, hence the default to True.
                 var_value = os.environ.get(var, "true")
-                # `os.environ` is a dict whose all values are strings. Here we normalize these string into
-                # booleans. If we can't, we fallback to True, in the same spirit as
-                # above.
+                # `os.environ` is a dict whose all values are strings. Here we normalize
+                # these string into booleans. If we can't, we fallback to True, in the
+                # same spirit as above.
                 var_boolean = RawConfigParser.BOOLEAN_STATES.get(
                     var_value.lower(), True
                 )
@@ -222,7 +220,7 @@ class ColorOption(ExtraOption):
                 value = True in colorize_from_env
 
         # There is an undocumented color flag in context:
-        # https://github.com/pallets/click/blob/65eceb08e392e74dcc761be2090e951274ccbe36/src/click/globals.py#L56-L69
+        # https://github.com/pallets/click/blob/65eceb0/src/click/globals.py#L56-L69
         ctx.color = value
 
         if not value:
@@ -260,204 +258,6 @@ class ColorOption(ExtraOption):
         )
 
 
-color_option = partial(option, cls=ColorOption)
-"""Decorator for ``ColorOption``."""
-
-
-class VersionOption(ExtraOption):
-    """Prints the colored version of the CLI.
-
-    This is a [copy of the standard ``@click.version_option()`` decorator](https://github.com/pallets/click/blob/dc918b48fb9006be683a684b42cc7496ad649b83/src/click/decorators.py#L399-L466)
-    that has been made into a class to allow it to be used with declarative `params=` argument
-    (fixes [Click #2324 issue](https://github.com/pallets/click/issues/2324)).
-    """
-
-    version: str | None = None
-    package_name: str | None = None
-    prog_name: str | None = None
-    message: str = _("%(prog)s, version %(version)s")
-
-    def guess_package_name(self):
-        package_name = None
-
-        frame = inspect.currentframe()
-        f_back = frame.f_back if frame is not None else None
-        f_globals = f_back.f_globals if f_back is not None else None
-        # break reference cycle
-        # https://docs.python.org/3/library/inspect.html#the-interpreter-stack
-        del frame
-
-        if f_globals is not None:
-            package_name = f_globals.get("__name__")
-
-            if package_name == "__main__":
-                package_name = f_globals.get("__package__")
-
-            if package_name:
-                package_name = package_name.partition(".")[0]
-
-        return package_name
-
-    def print_version(
-        self,
-        ctx: Context,
-        param: Parameter,
-        value: bool,
-        capture_output: bool = False,
-    ) -> str | None:
-        """Prints version and exits.
-
-        Standard callback with an extra ``capture_output`` parameter which returns the
-        output string instead of printing the (colored) version to the console.
-        """
-        if not value or ctx.resilient_parsing:
-            return None
-
-        if self.prog_name is None:
-            self.prog_name = ctx.find_root().info_name
-
-        if self.version is None and self.package_name is not None:
-            metadata: ModuleType | None
-
-            try:
-                from importlib import metadata  # type: ignore
-            except ImportError:
-                # Python < 3.8
-                import importlib_metadata as metadata  # type: ignore
-
-            try:
-                self.version = metadata.version(self.package_name)  # type: ignore
-            except metadata.PackageNotFoundError:  # type: ignore
-                raise RuntimeError(
-                    f"{self.package_name!r} is not installed. Try passing"
-                    " 'package_name' instead."
-                ) from None
-
-        if self.version is None:
-            raise RuntimeError(
-                f"Could not determine the version for {self.package_name!r} automatically."
-            )
-
-        output = self.message % {
-            "prog": self.prog_name,
-            "package": self.package_name,
-            "version": self.version,
-        }
-
-        if capture_output:
-            return output
-
-        echo(output, color=ctx.color)
-        ctx.exit()
-
-    def __init__(
-        self,
-        param_decls: Iterable[str] | None = None,
-        version: str | None = None,
-        package_name: str | None = None,
-        prog_name: str | None = None,
-        message: str | None = None,
-        print_env_info: bool = False,
-        version_style=Style(fg="green"),
-        package_name_style=default_theme.invoked_command,
-        prog_name_style=default_theme.invoked_command,
-        message_style=None,
-        env_info_style=Style(fg="bright_black"),
-        is_flag=True,
-        expose_value=False,
-        is_eager=True,
-        help=_("Show the version and exit."),
-        **kwargs,
-    ) -> None:
-        """For other params see Click's ``version_option`` decorator:
-        https://click.palletsprojects.com/en/8.1.x/api/#click.version_option.
-
-        :param param_decls: _description_, defaults to None
-        :type param_decls: _type_, optional
-        :param version: _description_, defaults to None
-        :type version: _type_, optional
-        :param package_name: _description_, defaults to None
-        :type package_name: t.Optional[str], optional
-        :param prog_name: _description_, defaults to None
-        :type prog_name: t.Optional[str], optional
-        :param message: _description_, defaults to "%(prog)s, version %(version)s"
-        :type message: str, optional
-        :param print_env_info: _description_, defaults to False
-        :type print_env_info: bool, optional
-        :param version_style: adds environment info at the end of the message. Useful to gather user's details for troubleshooting. Defaults to ``Style(fg="green")``.
-        :type version_style: _type_, optional
-        :param package_name_style: style of the ``version``. Defaults to ``default_theme.invoked_command``.
-        :type package_name_style: _type_, optional
-        :param prog_name_style: style of the ``prog_name``. Defaults to ``default_theme.invoked_command``.
-        :type prog_name_style: _type_, optional
-        :param message_style: default style of the ``message`` parameter. Defaults to ``None``.
-        :type message_style: _type_, optional
-        :param env_info_style: style of the environment info. Defaults to ``Style(fg="bright_black")``.
-        :type env_info_style: _type_, optional
-        :param is_flag: _description_, defaults to True
-        :type is_flag: bool, optional
-        :param expose_value: _description_, defaults to False
-        :type expose_value: bool, optional
-        :param is_eager: _description_, defaults to True
-        :type is_eager: bool, optional
-        :param help: _description_, defaults to "Show the version and exit."
-        :type help: str, optional
-        """
-        if not param_decls:
-            param_decls = ("--version",)
-
-        kwargs.setdefault("callback", self.print_version)
-
-        self.version = version
-        self.package_name = package_name
-        self.prog_name = prog_name
-        if message:
-            self.message = message
-
-        if self.version is None and self.package_name is None:
-            self.package_name = self.guess_package_name()
-
-        # XXX Temporarily skip displaying environment details for Python >= 3.10 while we wait for
-        # https://github.com/mahmoud/boltons/issues/294 to be released upstream.
-        if print_env_info and sys.version_info[:2] < (3, 10):
-            from boltons.ecoutils import get_profile
-
-            env_info = "\n" + str(get_profile(scrub=True))
-            if env_info_style:
-                env_info = env_info_style(env_info)
-            self.message += env_info
-
-        colorized_message = ""
-        for part in re.split(r"(%\(version\)s|%\(package\)s|%\(prog\)s)", self.message):
-            # Skip empty strings.
-            if not part:
-                continue
-            if part == "%(package)s" and package_name_style:
-                part = package_name_style(part)
-            elif part == "%(prog)s" and prog_name_style:
-                part = prog_name_style(part)
-            elif part == "%(version)s" and version_style:
-                part = version_style(part)
-            elif message_style:
-                part = message_style(part)
-            colorized_message += part
-        if colorized_message:
-            self.message = colorized_message
-
-        super().__init__(
-            param_decls=param_decls,
-            is_flag=is_flag,
-            expose_value=expose_value,
-            is_eager=is_eager,
-            help=help,
-            **kwargs,
-        )
-
-
-version_option = partial(option, cls=VersionOption)
-"""Decorator for ``VersionOption``."""
-
-
 class HelpOption(ExtraOption):
     @staticmethod
     def print_help(ctx: Context, param: Parameter, value: bool) -> None:
@@ -492,15 +292,11 @@ class HelpOption(ExtraOption):
         )
 
 
-help_option = partial(option, cls=HelpOption)
-"""Decorator for ``HelpOption``."""
-
-
 class ExtraHelpColorsMixin:
     """Adds extra-keywords highlighting to Click commands.
 
-    This mixin for `click.core.Command`-like classes intercepts the top-level
-    helper-generation method to initialize the formatter with dynamic settings.
+    This mixin for `click.core.Command`-like classes intercepts the top-level helper-
+    generation method to initialize the formatter with dynamic settings.
 
     This is implemented here to get access to the global context.
     """
@@ -518,9 +314,9 @@ class ExtraHelpColorsMixin:
         cli_names.add(ctx.command_path)
         command = ctx.command
 
-        # Will fetch command's metavar (i.e. the "[OPTIONS]" after the CLI name in "Usage:") and dig
-        # into subcommands to get subcommand_metavar: ("COMMAND1 [ARGS]...
-        # [COMMAND2 [ARGS]...]...").
+        # Will fetch command's metavar (i.e. the "[OPTIONS]" after the CLI name in
+        # "Usage:") and dig into subcommands to get subcommand_metavar:
+        # ("COMMAND1 [ARGS]... [COMMAND2 [ARGS]...]...").
         metavars.update(command.collect_usage_pieces(ctx))
 
         # Get subcommands and their aliases.
@@ -546,16 +342,16 @@ class ExtraHelpColorsMixin:
         # Split between shorts and long options
         long_options: set[str] = set()
         short_options: set[str] = set()
-        for option in options:
+        for option_name in options:
             # TODO: reuse ctx._opt_prefixes for finer match?
-            # Short options no longer than 2 characters
-            # (example: "-D", "/d", "/?", "+w", "-w", "f_", "_f", ...)
-            if len(option) <= 2:
-                short_options.add(option)
-            # Any other is considered a long options
-            # (example: "--debug", "--c", "-otest", "---debug", "-vvvv, "++foo", "/debug", "from_", "_from", ...)
+            # Short options no longer than 2 characters like "-D", "/d", "/?", "+w",
+            # "-w", "f_", "_f", ...)
+            if len(option_name) <= 2:
+                short_options.add(option_name)
+            # Any other is considered a long options. Like: "--debug", "--c", "-otest",
+            # "---debug", "-vvvv, "++foo", "/debug", "from_", "_from", ...
             else:
-                long_options.add(option)
+                long_options.add(option_name)
 
         return (
             cli_names,
@@ -586,6 +382,16 @@ class ExtraHelpColorsMixin:
         return super().format_help(ctx, formatter)
 
 
+def escape_for_help_sceen(text: str) -> str:
+    """Escape a text to be used in a regural expression to match help screen.
+
+    Like ``re.escape``, but allows any number of optional blank characters (line
+    returns, spaces, tabs) after a dash, to accounts for text wrapping rules and
+    columnar layout.
+    """
+    return re.escape(text).replace("-", "-\\s*")
+
+
 class HelpExtraFormatter(HelpFormatter):
     """Extends Cloup's custom HelpFormatter to highlights options, choices, metavars and
     default values.
@@ -663,14 +469,13 @@ class HelpExtraFormatter(HelpFormatter):
         is good enough. After all, help screens are not consumed by machine but are
         designed for humans.
         """
-        # Highlight "(Deprecated)" or "(DEPRECATED)" flag, as set by either:
+        # Highlight " (Deprecated)" or " (DEPRECATED)" labels, as set by either:
         # https://github.com/pallets/click/blob/ef11be6e49e19a055fe7e5a89f0f1f4062c68dba/tests/test_commands.py#L345
         # https://github.com/janluke/cloup/blob/c29fa051ed405856ed8bc2dbd733f9df2c8e6418/cloup/formatting/_formatter.py#L188
         help_text = re.sub(
             rf"""
-            (\s)                         # Any blank char.
-            (?P<warning>\(DEPRECATED\))  # The flag string.
-            (\s)                         # Any blank char.
+            (\s)                                      # Any blank char.
+            (?P<warning>{re.escape("(DEPRECATED)")})  # The flag string.
             """,
             self.colorize,
             help_text,
@@ -688,7 +493,7 @@ class HelpExtraFormatter(HelpFormatter):
                     \(                        # An opening parenthesis.
                     .*                        # Any string.
                 )
-                (?P<command_aliases>{alias})  # The alias.
+                (?P<command_aliases>{re.escape(alias)})  # The alias.
                 (
                     .*                        # Any string.
                     \)                        # A closing parenthesis.
@@ -704,7 +509,7 @@ class HelpExtraFormatter(HelpFormatter):
             help_text = re.sub(
                 rf"""
                 (\ \ )                        # 2 spaces (i.e. section indention).
-                (?P<subcommand>{subcommand})
+                (?P<subcommand>{re.escape(subcommand)})
                 (\s)                          # Any blank char.
                 """,
                 self.colorize,
@@ -721,7 +526,7 @@ class HelpExtraFormatter(HelpFormatter):
                 default:            # Starting content within the brackets.
                 \s+                 # Any number of blank chars.
             )
-            (?P<default_value>.+?)  # Greedy-matching of any string (including line returns).
+            (?P<default_value>.+?)  # Greedy-matching of any string and line returns.
             (?P<default_end>\])     # Square brackets closing.
             """,
             self.colorize,
@@ -733,9 +538,9 @@ class HelpExtraFormatter(HelpFormatter):
         for cli_name in self.cli_names:
             help_text = re.sub(
                 rf"""
-                (\s)                             # Any blank char.
-                (?P<invoked_command>{cli_name})  # The CLI name.
-                (\s)                             # Any blank char.
+                (\s)                                        # Any blank char.
+                (?P<invoked_command>{re.escape(cli_name)})  # The CLI name.
+                (\s)                                        # Any blank char.
                 """,
                 self.colorize,
                 help_text,
@@ -743,7 +548,9 @@ class HelpExtraFormatter(HelpFormatter):
             )
 
         # Highligh sections.
-        # XXX Duplicates Cloup's job, with the only subtlety of not highlighting the trailing semicolon.
+        # XXX Duplicates Cloup's job, with the only subtlety of not highlighting the
+        # trailing semicolon.
+        #
         # help_text = re.sub(
         #     r"""
         #     ^                       # Beginning of a line preceded by a newline.
@@ -763,20 +570,12 @@ class HelpExtraFormatter(HelpFormatter):
             (sorted(self.metavars, reverse=True), "metavar"),
         ):
             for keyword in matching_keywords:
-                # Regexp is verbose, so we need to escape spaces.
-                keyword = keyword.replace(" ", r"\ ")
-                # Accounts for text wrapping of options after a dash.
-                keyword = keyword.replace("-", "-\\s*")
-                # Allow metavars to have square braquets "[]" and dots (like command's
-                # options_metavar).
-                keyword = (
-                    keyword.replace("[", "\\[").replace("]", "\\]").replace(".", "\\.")
-                )
+                keyword = escape_for_help_sceen(keyword)
                 help_text = re.sub(
                     rf"""
                     ([               # A keyword is preceded with either:
                         \s           # - a blank char
-                        \[           # - an opening square bracket (like in choice strings)
+                        \[           # - an opening square bracket (as in choice string)
                         \|           # - a pipe (again like in choice strings)
                         \(           # - an opening parenthesis
                     ])
