@@ -1,5 +1,4 @@
 # Copyright Kevin Deldycke <kevin@deldycke.com> and contributors.
-# All Rights Reserved.
 #
 # This program is Free Software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -19,14 +18,14 @@
 
 from __future__ import annotations
 
+import os
 import sys
 from collections.abc import MutableMapping
 from configparser import ConfigParser, ExtendedInterpolation
 from enum import Enum
-from functools import partial, reduce
+from functools import reduce
 from gettext import gettext as _
 from operator import getitem, methodcaller
-from os.path import sep
 from pathlib import Path
 from typing import Any, Iterable, Sequence
 from unittest.mock import patch
@@ -39,21 +38,32 @@ else:
 if sys.version_info >= (3, 11):
     import tomllib
 else:
-    import tomli as tomllib
+    import tomli as tomllib  # type: ignore[import]
 
 import commentjson as json
 import requests
 import xmltodict
 import yaml
 from boltons.iterutils import flatten, remap
+from boltons.pathutils import shrinkuser
 from boltons.urlutils import URL
-from click import BOOL, FLOAT, INT, STRING, UNPROCESSED, UUID, Option, Parameter
+from click import (
+    BOOL,
+    FLOAT,
+    INT,
+    STRING,
+    UNPROCESSED,
+    UUID,
+    Option,
+    Parameter,
+    echo,
+    get_app_dir,
+    get_current_context,
+)
 from click import Path as ClickPath
-from click import echo, get_app_dir, get_current_context
 from click.core import ParameterSource
-from cloup import Choice, DateTime, File, FloatRange, IntRange
+from cloup import Choice, DateTime, File, FloatRange, IntRange, Style
 from cloup import Tuple as CloupTuple
-from cloup import option
 from mergedeep import merge
 from tabulate import tabulate
 from wcmatch.glob import (
@@ -67,9 +77,10 @@ from wcmatch.glob import (
     iglob,
 )
 
+from .colorize import KO, OK, default_theme
 from .logging import logger
 from .parameters import ExtraOption
-from .platform import is_windows
+from .platforms import is_windows
 
 
 class Formats(Enum):
@@ -91,18 +102,37 @@ class ParamStructure:
 
     Structures are represented by a tree-like ``dict``.
 
-    Access to a node is available using a serialized path string composed of the keys to descend to that node,
-    separated by a dot ``.``.
+    Access to a node is available using a serialized path string composed of the keys to
+    descend to that node, separated by a dot ``.``.
     """
 
-    ignored_params: Iterable[str] = tuple()
-
     SEP: str = "."
-    """Use a dot ``.`` as a separator between levels of the tree-like parameter structure."""
+    """Use a dot ``.`` as a separator between levels of the tree-like parameter
+    structure."""
 
-    def __init__(self, *args, ignored_params: Iterable[str] | None = None, **kwargs):
-        if ignored_params:
-            self.ignored_params = ignored_params
+    DEFAULT_EXCLUDE_PARAMS: Iterable[str] = (
+        "config",
+        "help",
+        "show_params",
+        "version",
+    )
+    """List of root parameters to exclude from configuration by default:
+
+    - ``-C``/``--config`` option, which cannot be used to recursively load another
+      configuration file.
+    - ``--help``, as it makes no sense to have the configurable file always
+      forces a CLI to show the help and exit.
+    - ``--show-params`` flag, which is like ``--help`` and stops the CLI execution.
+    - ``--version``, which is not a configurable option *per-se*.
+    """
+
+    def __init__(self, *args, exclude_params: Iterable[str] | None = None, **kwargs):
+        """Force the blocklist with paramerers provided by the user.
+
+        Else, let the cached ``self.exclude_params`` property compute it.
+        """
+        if exclude_params is not None:
+            self.exclude_params = exclude_params
 
         super().__init__(*args, **kwargs)
 
@@ -128,7 +158,8 @@ class ParamStructure:
 
     def _flatten_tree_dict_gen(self, tree_dict, parent_key):
         """
-        Source: https://www.freecodecamp.org/news/how-to-flatten-a-dictionary-in-python-in-4-different-ways/
+        `Source of this snippet
+        <https://www.freecodecamp.org/news/how-to-flatten-a-dictionary-in-python-in-4-different-ways/>`_.
         """
         for k, v in tree_dict.items():
             new_key = f"{parent_key}{self.SEP}{k}" if parent_key else k
@@ -145,11 +176,14 @@ class ParamStructure:
         return dict(self._flatten_tree_dict_gen(tree_dict, parent_key))
 
     def walk_params(self):
-        """Generator yielding all CLI parameters from top-level to subcommands.
+        """Generates an unfiltered list of all CLI parameters.
+
+        Everything is included, from top-level to subcommands, from options to
+        arguments.
 
         Returns a 2-elements tuple:
-        - the first being a tuple of keys leading to the parameter
-        - the second being the parameter object itself
+            - the first being a tuple of keys leading to the parameter
+            - the second being the parameter object itself
         """
         ctx = get_current_context()
         cli = ctx.find_root().command
@@ -160,9 +194,8 @@ class ParamStructure:
 
         # Global, top-level options shared by all subcommands.
         for p in cli.params:
-            if p.name not in self.ignored_params:
-                top_level_params.add(p.name)
-                yield (cli.name, p.name), p
+            top_level_params.add(p.name)
+            yield (cli.name, p.name), p
 
         # Subcommand-specific options.
         if hasattr(cli, "commands"):
@@ -174,14 +207,13 @@ class ParamStructure:
                     )
 
                 for p in cmd.params:
-                    if p.name not in self.ignored_params:
-                        yield (cli.name, cmd_id, p.name), p
+                    yield (cli.name, cmd_id, p.name), p
 
     def get_param_type(self, param):
         """Get the Python type of a Click parameter.
 
-        See the list of `custom types provided by Click
-        <https://click.palletsprojects.com/en/8.1.x/api/?highlight=intrange#types>`_.
+        See the list of
+        `custom types provided by Click <https://click.palletsprojects.com/en/8.1.x/api/?highlight=intrange#types>`_.
         """
         if param.multiple or param.nargs != 1:
             return list
@@ -220,13 +252,33 @@ class ParamStructure:
             f"Can't guess the appropriate Python type of {param!r} parameter."
         )
 
+    @cached_property
+    def exclude_params(self) -> Iterable[str]:
+        """List of parameter IDs to exclude from the parameter structure.
+
+        Elements of this list are expected to be the fully-qualified ID of the
+        parameter, i.e. the dot-separated ID that is prefixed by the CLI name.
+
+        It's been made into a property to allow for a last-minute call to the current
+        context to fetch the CLI name.
+        """
+        ctx = get_current_context()
+        cli = ctx.find_root().command
+        return [f"{cli.name}{self.SEP}{p}" for p in self.DEFAULT_EXCLUDE_PARAMS]
+
     def build_param_trees(self) -> None:
-        """Build all parameters tree structure in one go and cache them."""
+        """Build all parameters tree structure in one go and cache them.
+
+        This removes parameters whose fully-qualified IDs are in the ``exclude_params``
+        blocklist.
+        """
         template: dict[str, Any] = {}
         types: dict[str, Any] = {}
         objects: dict[str, Any] = {}
 
         for keys, param in self.walk_params():
+            if self.SEP.join(keys) in self.exclude_params:
+                continue
             merge(template, self.init_tree_dict(*keys))
             merge(types, self.init_tree_dict(*keys, leaf=self.get_param_type(param)))
             merge(objects, self.init_tree_dict(*keys, leaf=param))
@@ -282,41 +334,40 @@ class ConfigOption(ExtraOption, ParamStructure):
         metavar="CONFIG_PATH",
         type=STRING,
         help=_(
-            "Location of the configuration file. Supports glob pattern of local path and remote URL."
+            "Location of the configuration file. Supports glob pattern of local "
+            "path and remote URL."
         ),
         is_eager=True,
         expose_value=False,
         formats=tuple(Formats),
         roaming=True,
         force_posix=False,
-        ignored_params=(
-            "help",
-            "version",
-            "config",
-            "show_params",
-        ),
+        exclude_params=None,
         strict=False,
         **kwargs,
     ):
-        """A [``wcmatch.glob``
-        pattern](https://facelessuser.github.io/wcmatch/glob/#syntax).
+        """Takes as input either a `wcmatch.glob local-file pattern.
 
-        - ``is_eager`` is active by default so the config option's ``callback`` gets the opportunity to set the
-            ``default_map`` values before the other options use them.
+        <https://facelessuser.github.io/wcmatch/glob/#syntax>`_ or an URL.
 
-        - ``formats`` is the ordered list of formats that the configuration file will be tried to be read with. Can be a single one.
+        - ``is_eager`` is active by default so the config option's ``callback``
+          gets the opportunity to set the ``default_map`` values before the
+          other options use them.
 
-        - ``roaming`` and ``force_posix`` are [fed to ``click.get_app_dir()``](https://click.palletsprojects.com/en/8.1.x/api/#click.get_app_dir) to setup the default configuration
-            folder.
+        - ``formats`` is the ordered list of formats that the configuration
+          file will be tried to be read with. Can be a single one.
 
-        - ``ignored_params`` is a list of options to ignore by the configuration parser. Defaults to:
-            - ``--help``, as it makes no sense to have the configurable file always forces a CLI to show the help and exit.
-            - ``--version``, which is not a configurable option *per-se*.
-            - ``-C``/``--config`` option, which cannot be used to recursively load another configuration file (yet?).
-            - ``--show-params`` flag, which is like ``--help`` and stops the CLI execution.
+        - ``roaming`` and ``force_posix`` are `fed to click.get_app_dir()
+          <https://click.palletsprojects.com/en/8.1.x/api/#click.get_app_dir>`_
+          to setup the default configuration folder.
+
+        - ``exclude_params`` is a list of options to ignore by the
+          configuration parser. Defaults to
+          ``ParamStructure.DEFAULT_EXCLUDE_PARAMS``.
 
         - ``strict``
-            - If ``True``, raise an error if the configuration file contain unrecognized content.
+            - If ``True``, raise an error if the configuration file contain
+              unrecognized content.
             - If ``False``, silently ignore unsupported configuration option.
         """
         if not param_decls:
@@ -332,7 +383,8 @@ class ConfigOption(ExtraOption, ParamStructure):
         self.force_posix = force_posix
         kwargs.setdefault("default", self.default_pattern)
 
-        self.ignored_params = ignored_params
+        if exclude_params is not None:
+            self.exclude_params = exclude_params
 
         self.strict = strict
 
@@ -351,12 +403,15 @@ class ConfigOption(ExtraOption, ParamStructure):
     def default_pattern(self) -> str:
         """Returns the default pattern used to search for the configuration file.
 
-        Defaults to ``/<app_dir>/*.{toml,yaml,yml,json,ini,xml}``.
+        Defaults to ``/<app_dir>/*.{toml,yaml,yml,json,ini,xml}``. Where
+        ``<app_dir>`` is produced by the `clickget_app_dir() method
+        <https://click.palletsprojects.com/en/8.1.x/api/#click.get_app_dir>`_.
+        The result depends on OS and is influenced by the ``roaming`` and
+        ``force_posix`` properties of this instance.
 
-        ``<app_dir>`` is produced by [`clickget_app_dir()` method](https://click.palletsprojects.com/en/8.1.x/api/#click.get_app_dir).
-        The result depends on OS and is influenced by the ``roaming`` and ``force_posix`` properties of this instance.
+        In that folder, we're looking for any file matching the extensions
+        derived from the ``self.formats`` property:
 
-        In that folder, we're looking for any file matching the extensions derived from the ``self.formats`` property:
         - a simple ``*.ext`` pattern if only one format is set
         - an expanded ``*.{ext1,ext2,...}`` pattern if multiple formats are set
         """
@@ -374,25 +429,13 @@ class ConfigOption(ExtraOption, ParamStructure):
         else:
             # Use brace notation for multiple extension matching.
             ext_pattern = f"{{{','.join(extensions)}}}"
-        return f"{app_dir}{sep}*.{ext_pattern}"
-
-    @staticmethod
-    def compress_path(path: Path) -> Path:
-        """Reduces a path length by prefixing it with the `~` user's home prefix if
-        possible."""
-        if not is_windows():
-            try:
-                path = "~" / path.relative_to(Path.home())
-            except (RuntimeError, ValueError):
-                pass
-        return path
+        return f"{app_dir}{os.path.sep}*.{ext_pattern}"
 
     def get_help_record(self, ctx):
         """Replaces the default value by the pretty version of the configuration
         matching pattern."""
         # Pre-compute pretty_path to bypass infinite recursive loop on get_default.
-        default_path = Path(self.get_default(ctx))
-        pretty_path = self.compress_path(default_path)
+        pretty_path = shrinkuser(Path(self.get_default(ctx)))
         with patch.object(ConfigOption, "get_default") as mock_method:
             mock_method.return_value = pretty_path
             return super().get_help_record(ctx)
@@ -401,22 +444,27 @@ class ConfigOption(ExtraOption, ParamStructure):
         """Search on local file system or remote URL files matching the provided
         pattern.
 
-        Returns an iterator of raw content for each file/URL matching the pattern.
+        ``pattern`` is considered as an URL only if it is parseable as such and starts
+        with ``http://`` or ``https://``.
+
+        Returns an iterator of raw content for each file/URL matching the
+        pattern.
         """
         # Check if the pattern is an URL.
         location = URL(pattern)
         if location and location.scheme.lower() in ("http", "https"):
-            logger.debug(f"Fetch configuration from remote URL.")
+            logger.debug("Fetch configuration from remote URL.")
             with requests.get(location) as response:
                 if response.ok:
                     yield from (response.text,)
                     return
                 logger.warning(f"Can't download {location}: {response.reason}")
         else:
-            logger.debug(f"Pattern is not an URL.")
+            logger.debug("Pattern is not an URL.")
 
-        logger.debug(f"Search local file system.")
-        # wcmatch expect patterns to be written with unix-like syntax by default, even on Windows. See more details at:
+        logger.debug("Search local file system.")
+        # wcmatch expect patterns to be written with Unix-like syntax by default, even
+        # on Windows. See more details at:
         # https://facelessuser.github.io/wcmatch/glob/#windows-separators
         # https://github.com/facelessuser/wcmatch/issues/194
         if is_windows():
@@ -433,7 +481,7 @@ class ConfigOption(ExtraOption, ParamStructure):
         """Try to parse the provided content with each format in the order provided by
         the user.
 
-        A successful parsing in any format is supposed to return a dict. Any other
+        A successful parsing in any format is supposed to return a ``dict``. Any other
         result, including any raised exception, is considered a failure and the next
         format is tried.
         """
@@ -478,8 +526,9 @@ class ConfigOption(ExtraOption, ParamStructure):
     def load_ini_config(self, content):
         """Utility method to parse INI configuration file.
 
-        Internal convention is to use a dot (``.``, as set by ``self.SEP``) in section IDs as a separator between levels. This is a workaround
-        the limitation of INI format which doesn't allow for sub-sections.
+        Internal convention is to use a dot (``.``, as set by ``self.SEP``) in
+        section IDs as a separator between levels. This is a workaround
+        the limitation of ``INI`` format which doesn't allow for sub-sections.
 
         Returns a ready-to-use data structure.
         """
@@ -488,7 +537,6 @@ class ConfigOption(ExtraOption, ParamStructure):
 
         conf = {}
         for section_id in ini_config.sections():
-
             # Extract all options of the section.
             sub_conf = {}
             for option_id in ini_config.options(section_id):
@@ -508,14 +556,15 @@ class ConfigOption(ExtraOption, ParamStructure):
                 elif target_type == bool:
                     value = ini_config.getboolean(section_id, option_id)
 
-                # Types not natively supported by INI format are loaded as JSON-serialized
-                # strings.
+                # Types not natively supported by INI format are loaded as
+                # JSON-serialized strings.
                 elif target_type in (list, tuple, set, frozenset, dict):
                     value = json.loads(ini_config.get(section_id, option_id))
 
                 else:
                     raise ValueError(
-                        f"Conversion of {target_type} type for [{section_id}]:{option_id} INI config option."
+                        f"Conversion of {target_type} type for "
+                        f"[{section_id}]:{option_id} INI config option."
                     )
 
                 sub_conf[option_id] = value
@@ -569,12 +618,12 @@ class ConfigOption(ExtraOption, ParamStructure):
         """Fetch parameters values from configuration file and merge them with the
         defaults.
 
-        User configuration is merged to the context ``default_map``, as in:
-        https://click.palletsprojects.com/en/8.1.x/commands/#context-defaults
-
+        User configuration is
+        `merged to the context default_map as Click does <https://click.palletsprojects.com/en/8.1.x/commands/#context-defaults>`_.
+        
         This allow user's config to only overrides defaults. Values sets from direct
-        command line parameters, environment variables or interactive prompts, takes precedence
-        over any values from the config file.
+        command line parameters, environment variables or interactive prompts, takes
+        precedence over any values from the config file.
         """
         explicit_conf = ctx.get_parameter_source("config") in (
             ParameterSource.COMMANDLINE,
@@ -624,15 +673,19 @@ class ShowParamsOption(ExtraOption, ParamStructure):
     print information about the parameters that will be fed to the CLI.
     """
 
-    TABLE_HEADERS = [
-        "Parameter",
+    TABLE_HEADERS = (
         "ID",
+        "Class",
+        "Spec.",
         "Type",
+        "Allowed in conf?",
+        "Exposed",
         "Env. var.",
         "Default",
         "Value",
         "Source",
-    ]
+    )
+    """Hard-coded list of table headers."""
 
     def __init__(
         self,
@@ -641,7 +694,7 @@ class ShowParamsOption(ExtraOption, ParamStructure):
         expose_value=False,
         is_eager=True,
         help=_(
-            "Show all CLI parameters, their provenance, defaults, value, then exit."
+            "Show all CLI parameters, their provenance, defaults and value, then exit."
         ),
         **kwargs,
     ) -> None:
@@ -650,6 +703,9 @@ class ShowParamsOption(ExtraOption, ParamStructure):
 
         kwargs.setdefault("callback", self.print_params)
 
+        # Deactivate blocking of any parameter.
+        self.exclude_params = ()
+
         super().__init__(
             param_decls=param_decls,
             is_flag=is_flag,
@@ -661,14 +717,17 @@ class ShowParamsOption(ExtraOption, ParamStructure):
 
     @staticmethod
     def get_envvar(ctx, param: Parameter | Option):
-        """Emulates the retrieval or dynamic generation of a parameter's environment
+        """Emulates the retrieval and dynamic generation of a parameter's environment
         variable.
 
-        This code is a copy of what happens in ``click.core.Parameter.resolve_envvar_value()`` and
-        ``click.core.Option.resolve_envvar_value()`` as the logic is deeply embedded in Click's internals
-        and can't be independently used.
+        .. important::
+            This code is a copy of what happens in
+            ``click.core.Parameter.resolve_envvar_value()`` and
+            ``click.core.Option.resolve_envvar_value()`` as the logic is deeply
+            embedded in Click's internals and can't be independently used.
 
-        ..todo:: Contribute this to Click as slight refactor to DRY?
+        .. todo::
+            Contribute this to Click as slight refactor to DRY?
         """
         if param.envvar:
             return param.envvar
@@ -682,14 +741,22 @@ class ShowParamsOption(ExtraOption, ParamStructure):
         return None
 
     def print_params(self, ctx, param, value):
-        """Introspects current CLI ans list its parameters and metadata."""
+        """Introspects current CLI and list its parameters and metadata.
+
+        .. important::
+            Click doesn't keep a list of all parsed arguments and their origin.
+            So we need to emulate here what's happening during CLI invokation.
+            But can't even to that because the raw, pre-parsed arguments are
+            not available anywhere.
+
+            Our workaround consist in leveraging our custom
+            ``ExtraCommand``/``ExtraGroup`` classes, in which we are attaching
+            a ``click_extra.raw_args`` metadata entry to the context.
+        """
         # Exit early if the callback was processed but the option wasn't set.
         if not value:
             return
 
-        # Click doen't keep a list of all parsed arguments and their origin. We need to emulate what's happening
-        # during CLI invokation. The problem is even the raw arguments are now available somewhere. Our workaround
-        # consist in leveraging our ExtraCommand/ExtraGroup for this.
         if "click_extra.raw_args" in ctx.meta:
             raw_args = ctx.meta.get("click_extra.raw_args", [])
             logger.debug(f"click_extra.raw_args: {raw_args}")
@@ -699,9 +766,9 @@ class ShowParamsOption(ExtraOption, ParamStructure):
             parser = ctx.command.make_parser(ctx)
             opts, _, _ = parser.parse_args(args=raw_args)
 
-            # We call directly consume_value() instead of handle_parse_result() to prevent an
-            # embedded call to process_value(), as the later triggers the callback
-            # (and might terminate CLI execution).
+            # We call directly consume_value() instead of handle_parse_result() to
+            # prevent an embedded call to process_value(), as the later triggers the
+            # callback (and might terminate CLI execution).
             param_value, source = param.consume_value(ctx, opts)
 
             get_param_value = methodcaller("consume_value", ctx, opts)
@@ -709,7 +776,8 @@ class ShowParamsOption(ExtraOption, ParamStructure):
         else:
             logger.debug(f"click_extra.raw_args not in {ctx.meta}")
             logger.warning(
-                f"Cannot extract parameters values: {ctx.command} does not inherits from ExtraCommand."
+                f"Cannot extract parameters values: "
+                f"{ctx.command} does not inherits from ExtraCommand."
             )
 
             def vanilla_getter(param):
@@ -727,11 +795,16 @@ class ShowParamsOption(ExtraOption, ParamStructure):
             assert param.name == tree_keys[-1]
 
             param_value, source = get_param_value(param)
+            param_class = self.get_tree_value(self.params_objects, *tree_keys).__class__
+            param_spec = param.get_help_record(ctx)[0]
 
             line = (
-                self.get_tree_value(self.params_objects, *tree_keys),
-                path,
+                default_theme.invoked_command(path),
+                f"{param_class.__module__}.{param_class.__qualname__}",
+                param_spec,
                 param_type.__name__,
+                None,  # XXX TODO
+                OK if param.expose_value is True else KO,
                 self.get_envvar(ctx, param),
                 param.get_default(ctx),
                 param_value,
@@ -742,23 +815,18 @@ class ShowParamsOption(ExtraOption, ParamStructure):
         def sort_by_depth(line):
             """Sort parameters by depth first, then IDs, so that top-level parameters
             are kept to the top."""
-            param_path = line[1]
+            param_path = line[0]
             tree_keys = param_path.split(self.SEP)
             return len(tree_keys), param_path
 
+        header_style = Style(bold=True)
+        header_labels = map(header_style, self.TABLE_HEADERS)
+
         output = tabulate(
             sorted(table, key=sort_by_depth),
-            headers=self.TABLE_HEADERS,
+            headers=header_labels,
             tablefmt="rounded_outline",
             disable_numparse=True,
         )
         echo(output, color=ctx.color)
         ctx.exit()
-
-
-show_params_option = partial(option, cls=ShowParamsOption)
-"""Decorator for ``ShowParamsOption``."""
-
-
-config_option = partial(option, cls=ConfigOption)
-"""Decorator for ``ConfigOption``."""
