@@ -1,5 +1,4 @@
 # Copyright Kevin Deldycke <kevin@deldycke.com> and contributors.
-# All Rights Reserved.
 #
 # This program is Free Software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -18,7 +17,6 @@
 from __future__ import annotations
 
 import re
-import sys
 from textwrap import dedent
 
 import click
@@ -26,19 +24,24 @@ import cloup
 import pytest
 from boltons.strutils import strip_ansi
 from click import echo, secho, style
-from cloup import HelpTheme, Style, argument, command, option, option_group
-from pytest_cases import fixture, parametrize
+from cloup import HelpTheme, Style, argument, option, option_group
+from pytest_cases import parametrize
 
 from ..colorize import (
     HelpExtraFormatter,
     HelpExtraTheme,
-    color_option,
     default_theme,
     highlight,
-    version_option,
 )
-from ..commands import extra_command, extra_group
-from ..logging import LOG_LEVELS, logger, verbosity_option
+from ..decorators import (
+    color_option,
+    command,
+    extra_command,
+    extra_group,
+    help_option,
+    verbosity_option,
+)
+from ..logging import LOG_LEVELS, logger
 from .conftest import (
     command_decorators,
     default_debug_colored_log,
@@ -53,7 +56,7 @@ def test_theme_definition():
     logging primitives."""
     assert set(HelpTheme._fields).issubset(HelpExtraTheme._fields)
 
-    log_levels = {l.lower() for l in LOG_LEVELS}
+    log_levels = {level.lower() for level in LOG_LEVELS}
     assert log_levels.issubset(HelpExtraTheme._fields)
     assert log_levels.isdisjoint(HelpTheme._fields)
 
@@ -113,7 +116,7 @@ def test_only_full_word_highlight():
 @skip_windows_colors
 def test_keyword_collection(invoke):
     # Create a dummy Click CLI.
-    @extra_group()
+    @extra_group
     @option_group(
         "Group 1",
         option("-a", "--o1"),
@@ -125,7 +128,12 @@ def test_keyword_collection(invoke):
         option("--o4"),
     )
     @option("--test")
-    def color_cli1(o1, o2, o3, o4, test):
+    # Windows-style parameters.
+    @option("--boolean/--no-boolean", "-b/+B", is_flag=True)
+    @option("/debug;/no-debug")
+    # First option without an alias.
+    @option("--shout/--no-shout", " /-S", default=False)
+    def color_cli1(o1, o2, o3, o4, test, boolean, debug, shout):
         echo("It works!")
 
     @extra_command(params=None)
@@ -160,10 +168,20 @@ def test_keyword_collection(invoke):
         r"  \x1b\[36m--o4\x1b\[0m \x1b\[90mTEXT\x1b\[0m\n\n"
         r"\x1b\[94m\x1b\[1mOther options:\x1b\[0m\n"
         r"  \x1b\[36m--test\x1b\[0m \x1b\[90mTEXT\x1b\[0m\n"
+        r"  \x1b\[36m-b\x1b\[0m, \x1b\[36m--boolean\x1b\[0m / \x1b\[36m\+B\x1b\[0m,"
+        r" \x1b\[36m--no-boolean\x1b\[0m\n"
+        r"                            "
+        r"\x1b\[90m\[default: \x1b\[0m\x1b\[35mno-boolean\x1b\[0m\x1b\[90m\]\x1b\[0m\n"
+        r"  \x1b\[36m/debug\x1b\[0m; \x1b\[36m/no-debug\x1b\[0m"
+        r"         \x1b\[90m\[default:"
+        r" \x1b\[0m\x1b\[35mno-debug\x1b\[0m\x1b\[90m\]\x1b\[0m\n"
+        r"  \x1b\[36m--shout\x1b\[0m / \x1b\[36m-S\x1b\[0m, \x1b\[36m--no-shout\x1b\[0m"
+        r"  \x1b\[90m\[default: \x1b\[0m\x1b\[35mno-shout\x1b\[0m\x1b\[90m\]\x1b\[0m\n"
         rf"{default_options_colored_help}"
         r"\n"
         r"\x1b\[94m\x1b\[1mSubcommand group 1:\x1b\[0m\n"
-        r"  \x1b\[36mcommand1\x1b\[0m  CLI description with extra \x1b\[90mMY_VAR\x1b\[0m reference.\n"
+        r"  \x1b\[36mcommand1\x1b\[0m  CLI description with extra"
+        r" \x1b\[90mMY_VAR\x1b\[0m reference.\n"
         r"  \x1b\[36mcommand2\x1b\[0m\n\n"
         r"\x1b\[94m\x1b\[1mExtra commands:\x1b\[0m\n"
         r"  \x1b\[36mcommand3\x1b\[0m\n"
@@ -183,37 +201,35 @@ def test_keyword_collection(invoke):
     # CLI main group is invoked before sub-command.
     result = invoke(color_cli1, "command1", "--help", color=True)
     assert result.exit_code == 0
-    assert result.output == dedent(
-        f"""\
-        It works!
-        \x1b[94m\x1b[1mUsage: \x1b[0m\x1b[97mcolor-cli1 command1\x1b[0m \x1b[90m[OPTIONS]\x1b[0m [\x1b[36mMY_ARG\x1b[0m]...
-
-          CLI description with extra MY_VAR reference.
-
-        \x1b[94m\x1b[1mPositional arguments:\x1b[0m
-          [\x1b[36mMY_ARG\x1b[0m]...  Argument supports help.
-
-        \x1b[94m\x1b[1mOptions:\x1b[0m
-          \x1b[36m-h\x1b[0m, \x1b[36m--help\x1b[0m  Show this message and exit.
-        """
+    assert result.output == (
+        "It works!\n"
+        "\x1b[94m\x1b[1mUsage: \x1b[0m\x1b[97mcolor-cli1 command1\x1b[0m"
+        " \x1b[90m[OPTIONS]\x1b[0m [\x1b[36mMY_ARG\x1b[0m]...\n"
+        "\n"
+        "  CLI description with extra MY_VAR reference.\n"
+        "\n"
+        "\x1b[94m\x1b[1mPositional arguments:\x1b[0m\n"
+        "  [\x1b[36mMY_ARG\x1b[0m]...  Argument supports help.\n"
+        "\n"
+        "\x1b[94m\x1b[1mOptions:\x1b[0m\n"
+        "  \x1b[36m-h\x1b[0m, \x1b[36m--help\x1b[0m  Show this message and exit.\n"
     )
     assert not result.stderr
 
     # Standalone call to command: CLI main group is skipped.
     result = invoke(command1, "--help", color=True)
     assert result.exit_code == 0
-    assert result.output == dedent(
-        f"""\
-        \x1b[94m\x1b[1mUsage: \x1b[0m\x1b[97mcommand1\x1b[0m \x1b[90m[OPTIONS]\x1b[0m [\x1b[36mMY_ARG\x1b[0m]...
-
-          CLI description with extra MY_VAR reference.
-
-        \x1b[94m\x1b[1mPositional arguments:\x1b[0m
-          [\x1b[36mMY_ARG\x1b[0m]...  Argument supports help.
-
-        \x1b[94m\x1b[1mOptions:\x1b[0m
-          \x1b[36m-h\x1b[0m, \x1b[36m--help\x1b[0m  Show this message and exit.
-        """
+    assert result.output == (
+        "\x1b[94m\x1b[1mUsage: \x1b[0m\x1b[97mcommand1\x1b[0m"
+        " \x1b[90m[OPTIONS]\x1b[0m [\x1b[36mMY_ARG\x1b[0m]...\n"
+        "\n"
+        "  CLI description with extra MY_VAR reference.\n"
+        "\n"
+        "\x1b[94m\x1b[1mPositional arguments:\x1b[0m\n"
+        "  [\x1b[36mMY_ARG\x1b[0m]...  Argument supports help.\n"
+        "\n"
+        "\x1b[94m\x1b[1mOptions:\x1b[0m\n"
+        "  \x1b[36m-h\x1b[0m, \x1b[36m--help\x1b[0m  Show this message and exit.\n"
     )
     assert not result.stderr
 
@@ -234,71 +250,7 @@ def test_keyword_collection(invoke):
 
 
 @skip_windows_colors
-def test_standalone_version_option_with_env_info(invoke):
-    @click.group
-    @version_option(version="1.2.3.4", print_env_info=True)
-    def color_cli2():
-        echo("It works!")
-
-    # Test default colouring.
-    result = invoke(color_cli2, "--version", color=True)
-    assert result.exit_code == 0
-
-    regex_output = r"\x1b\[97mcolor-cli2\x1b\[0m, version \x1b\[32m1.2.3.4"
-    # XXX Temporarily skip displaying environment details for Python >= 3.10 while we wait for
-    # https://github.com/mahmoud/boltons/issues/294 to be released upstream.
-    if sys.version_info[:2] < (3, 10):
-        regex_output += r"\x1b\[0m\x1b\[90m\n{'.+'}"
-    regex_output += r"\x1b\[0m\n"
-    assert re.fullmatch(regex_output, result.output)
-
-    assert not result.stderr
-
-
-@pytest.mark.xfail(
-    strict=False, reason="version_option always displays click-extra version. See #176."
-)
-@skip_windows_colors
-@parametrize("cmd_decorator", command_decorators(no_groups=True))
-def test_standalone_version_option_without_env_info(invoke, cmd_decorator):
-    @cmd_decorator
-    @version_option(version="1.2.3.4", print_env_info=False)
-    def color_cli3():
-        echo("It works!")
-
-    # Test default colouring.
-    result = invoke(color_cli3, "--version", color=True)
-    assert result.exit_code == 0
-    assert (
-        result.output == "\x1b[97mcolor-cli3\x1b[0m, version \x1b[32m1.2.3.4\x1b[0m\n"
-    )
-    assert not result.stderr
-
-
-@skip_windows_colors
-@pytest.mark.parametrize(
-    "params", (None, "--help", "blah", ("--config", "random.toml"))
-)
-def test_integrated_version_option_precedence(invoke, params):
-    @extra_group(version="1.2.3.4")
-    def color_cli4():
-        echo("It works!")
-
-    result = invoke(color_cli4, "--version", params, color=True)
-    assert result.exit_code == 0
-
-    regex_output = r"\x1b\[97mcolor-cli4\x1b\[0m, version \x1b\[32m1.2.3.4"
-    # XXX Temporarily skip displaying environment details for Python >= 3.10 while we wait for
-    # https://github.com/mahmoud/boltons/issues/294 to be released upstream.
-    if sys.version_info[:2] < (3, 10):
-        regex_output += r"\x1b\[0m\x1b\[90m\n{'.+'}"
-    regex_output += r"\x1b\[0m\n"
-    assert re.fullmatch(regex_output, result.output)
-
-    assert not result.stderr
-
-
-@skip_windows_colors
+@parametrize("option_decorator", (color_option, color_option()))
 @pytest.mark.parametrize(
     "param,expecting_colors",
     (
@@ -309,14 +261,14 @@ def test_integrated_version_option_precedence(invoke, params):
         (None, True),
     ),
 )
-def test_standalone_color_option(invoke, param, expecting_colors):
+def test_standalone_color_option(invoke, option_decorator, param, expecting_colors):
     """Check color option values, defaults and effects on all things colored, including
     verbosity option."""
 
     @click.command
-    @verbosity_option()
-    @color_option()
-    def color_cli5():
+    @verbosity_option
+    @option_decorator
+    def standalone_color():
         echo(Style(fg="yellow")("It works!"))
         echo("\x1b[0m\x1b[1;36mArt\x1b[46;34m\x1b[0m")
         echo(style("Run command.", fg="magenta"))
@@ -324,11 +276,11 @@ def test_standalone_color_option(invoke, param, expecting_colors):
         print(style("print() bypass Click.", fg="blue"))
         secho("Done.", fg="green")
 
-    result = invoke(color_cli5, param, "--verbosity", "DEBUG", color=True)
+    result = invoke(standalone_color, param, "--verbosity", "DEBUG", color=True)
     assert result.exit_code == 0
 
     if expecting_colors:
-        assert result.output == (
+        assert result.stdout == (
             "\x1b[33mIt works!\x1b[0m\n"
             "\x1b[0m\x1b[1;36mArt\x1b[46;34m\x1b[0m\n"
             "\x1b[35mRun command.\x1b[0m\n"
@@ -340,7 +292,7 @@ def test_standalone_color_option(invoke, param, expecting_colors):
             "\x1b[33mwarning: \x1b[0mProcessing...\n"
         )
     else:
-        assert result.output == (
+        assert result.stdout == (
             "It works!\n"
             "Art\n"
             "Run command.\n"
@@ -352,37 +304,6 @@ def test_standalone_color_option(invoke, param, expecting_colors):
         )
 
 
-@skip_windows_colors
-def test_color_option_precedence(invoke):
-    """--no-color has an effect on --version, if placed in the right order.
-
-    Eager parameters are evaluated in the order as they were provided on the command
-    line by the user as expleined in:
-    https://click.palletsprojects.com/en/8.0.x/advanced/#callback-evaluation-order
-
-    ..todo:
-
-        Maybe have the possibility to tweak CLI callback evaluation order so we can
-        let the user to have the NO_COLOR env set to allow for color-less --version output.
-    """
-
-    @click.command
-    @color_option()
-    @version_option(version="2.1.9")
-    def color_cli6():
-        echo(Style(fg="yellow")("It works!"))
-
-    result = invoke(color_cli6, "--no-color", "--version", "command1", color=True)
-    assert result.exit_code == 0
-    assert result.output == "color-cli6, version 2.1.9\n"
-    assert not result.stderr
-
-    result = invoke(color_cli6, "--version", "--no-color", "command1", color=True)
-    assert result.exit_code == 0
-    assert result.output == "\x1b[97mcolor-cli6\x1b[0m, version \x1b[32m2.1.9\x1b[0m\n"
-    assert not result.stderr
-
-
 @skip_windows_colors
 @pytest.mark.parametrize(
     "env,env_expect_colors",
@@ -410,7 +331,7 @@ def test_no_color_env_convention(
     invoke, env, env_expect_colors, param, param_expect_colors
 ):
     @click.command
-    @color_option()
+    @color_option
     def color_cli7():
         echo(Style(fg="yellow")("It works!"))
 
@@ -441,7 +362,7 @@ def test_no_color_env_convention(
     ),
 )
 def test_integrated_color_option(invoke, param, expecting_colors):
-    @extra_group()
+    @extra_group
     def color_cli8():
         echo(Style(fg="yellow")("It works!"))
         echo("\x1b[0m\x1b[1;36mArt\x1b[46;34m\x1b[0m")
@@ -548,3 +469,39 @@ def test_substring_highlighting(substrings, expected, ignore_case):
         ignore_case=ignore_case,
     )
     assert result == expected
+
+
+@parametrize(
+    "cmd_decorator, cmd_type",
+    # Skip click extra's commands, as help option is already part of the default.
+    command_decorators(no_extra=True, with_types=True),
+)
+@parametrize("option_decorator", (help_option, help_option()))
+def test_standalone_help_option(invoke, cmd_decorator, cmd_type, option_decorator):
+    @cmd_decorator
+    @option_decorator
+    def standalone_help():
+        echo("It works!")
+
+    result = invoke(standalone_help, "--help")
+    assert result.exit_code == 0
+    assert not result.stderr
+
+    if "group" in cmd_type:
+        assert result.stdout == dedent(
+            """\
+            Usage: standalone-help [OPTIONS] COMMAND [ARGS]...
+
+            Options:
+              -h, --help  Show this message and exit.
+            """
+        )
+    else:
+        assert result.stdout == dedent(
+            """\
+            Usage: standalone-help [OPTIONS]
+
+            Options:
+              -h, --help  Show this message and exit.
+            """
+        )
