@@ -1,5 +1,4 @@
 # Copyright Kevin Deldycke <kevin@deldycke.com> and contributors.
-# All Rights Reserved.
 #
 # This program is Free Software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -18,12 +17,12 @@
 from __future__ import annotations
 
 import re
-import sys
 from os.path import sep
 from pathlib import Path
 
 import click
 import pytest
+from boltons.pathutils import shrinkuser
 from click import (
     BOOL,
     FLOAT,
@@ -36,15 +35,22 @@ from click import (
     File,
     FloatRange,
     IntRange,
+    Tuple,
+    echo,
+    get_app_dir,
 )
 from click import Path as ClickPath
-from click import Tuple, echo, get_app_dir
 from cloup import argument, option
+from pytest_cases import parametrize
 from tabulate import tabulate
 
-from ..commands import extra_command, extra_group
-from ..config import ConfigOption, ShowParamsOption, config_option
-from .conftest import default_debug_uncolored_log
+from ..colorize import escape_for_help_sceen
+from ..config import ConfigOption, ShowParamsOption
+from ..decorators import config_option, extra_command, extra_group, show_params_option
+from .conftest import (
+    command_decorators,
+    default_debug_uncolored_log,
+)
 
 DUMMY_TOML_FILE = """
     # Comment
@@ -190,7 +196,7 @@ all_config_formats = pytest.mark.parametrize(
 )
 
 
-@extra_group()
+@extra_group
 @option("--dummy-flag/--no-flag")
 @option("--my-list", multiple=True)
 def config_cli1(dummy_flag, my_list):
@@ -223,18 +229,15 @@ def test_conf_default_path(invoke):
     assert result.exit_code == 0
 
     # OS-specific path.
-    default_path = ConfigOption.compress_path(
+    default_path = shrinkuser(
         Path(get_app_dir("config-cli1")) / "*.{toml,yaml,yml,json,ini,xml}"
     )
 
     # Make path string compatible with regexp.
-    path_regexp = (
-        str(default_path)
-        .replace("\\", "\\\\")
-        .replace("*", r"\*")
-        .replace("-", r"-\s*")
+    assert re.search(
+        rf"\[default:\s+{escape_for_help_sceen(str(default_path))}\]",
+        result.output,
     )
-    assert re.search(rf"\[default:\s+{path_regexp}\]", result.output)
 
 
 def test_conf_not_exist(invoke):
@@ -263,9 +266,13 @@ def test_conf_not_file(invoke):
     )
 
 
-def test_conf_auto_types(invoke, create_config):
-    """Check the conf type and structure is properly derived from CLI options."""
+@parametrize("option_decorator", (config_option, config_option()))
+def test_conf_auto_types(invoke, create_config, option_decorator):
+    """Check the conf type and structure is properly derived from CLI options.
 
+    Also covers the tests of the standalone ``@config_option`` decorator in all its
+    flavors.
+    """
     @click.command
     @option("--flag1/--no-flag1")
     @option("--flag2", is_flag=True)
@@ -290,7 +297,7 @@ def test_conf_auto_types(invoke, create_config):
     @option("--list1", multiple=True)
     @argument("file_arg1", type=File("w"))
     @argument("file_arg2", type=File("w"), nargs=-1)
-    @config_option()
+    @option_decorator
     def config_cli2(
         flag1,
         flag2,
@@ -392,8 +399,7 @@ def test_conf_auto_types(invoke, create_config):
 
 def test_strict_conf(invoke, create_config):
     """Same test as the one shown in the readme, but in strict validation mode."""
-
-    @click.group(context_settings={"show_default": True})
+    @click.group
     @option("--dummy-flag/--no-flag")
     @option("--my-list", multiple=True)
     @config_option(strict=True)
@@ -444,7 +450,6 @@ def test_conf_file_overrides_defaults(
     conf_url = httpserver.url_for(f"/{conf_name}")
 
     for conf_path in conf_filepath, conf_url:
-
         result = invoke(
             config_cli1, "--config", str(conf_path), "default-command", color=False
         )
@@ -453,18 +458,13 @@ def test_conf_file_overrides_defaults(
             "dummy_flag = True\nmy_list = ('pip', 'npm', 'gem')\nint_parameter = 3\n"
         )
 
-        # Make path string compatible with regexp on Windows.
-        conf_path = str(conf_path).replace("\\", "\\\\")
         # Debug level has been activated by configuration file.
         debug_log = (
-            rf"Load configuration matching {conf_path}\n"
+            rf"Load configuration matching {re.escape(str(conf_path))}\n"
             r"debug: Verbosity set to DEBUG.\n"
             r"debug: \S+, version \S+\n"
+            r"debug: {.*}\n"
         )
-        # XXX Temporarily expect extra-env info for Python < 3.10 while we wait for
-        # https://github.com/mahmoud/boltons/issues/294 to be released upstream.
-        if sys.version_info[:2] < (3, 10):
-            debug_log += r"debug: {.*}\n"
         assert re.fullmatch(debug_log, result.stderr)
 
 
@@ -478,7 +478,6 @@ def test_auto_env_var_conf(invoke, create_config, httpserver, conf_name, conf_co
     conf_url = httpserver.url_for(f"/{conf_name}")
 
     for conf_path in conf_filepath, conf_url:
-
         conf_path = create_config(conf_name, conf_content)
         result = invoke(
             config_cli1,
@@ -507,7 +506,6 @@ def test_conf_file_overrided_by_cli_param(
     conf_url = httpserver.url_for(f"/{conf_name}")
 
     for conf_path in conf_filepath, conf_url:
-
         conf_path = create_config(conf_name, conf_content)
         result = invoke(
             config_cli1,
@@ -531,8 +529,51 @@ def test_conf_file_overrided_by_cli_param(
         assert result.stderr == f"Load configuration matching {conf_path.resolve()}\n"
 
 
-def test_show_params_option(invoke, create_config):
-    @extra_command()
+@parametrize(
+    "cmd_decorator",
+    # Skip click extra's commands, as show_params option is already part of the default.
+    command_decorators(no_groups=True, no_extra=True),
+)
+@parametrize("option_decorator", (show_params_option, show_params_option()))
+def test_standalone_show_params_option(invoke, cmd_decorator, option_decorator):
+    @cmd_decorator
+    @option_decorator
+    def show_params():
+        echo("It works!")
+
+    result = invoke(show_params, "--show-params")
+    assert result.exit_code == 0
+
+    table = [
+        (
+            "show-params.show_params",
+            "click_extra.config.ShowParamsOption",
+            "--show-params",
+            "bool",
+            "",
+            "✘",
+            "",
+            False,
+            "",
+            "COMMANDLINE",
+        ),
+    ]
+    output = tabulate(
+        table,
+        headers=ShowParamsOption.TABLE_HEADERS,
+        tablefmt="rounded_outline",
+        disable_numparse=True,
+    )
+    assert result.output == f"{output}\n"
+
+    assert result.stderr == (
+        "warning: Cannot extract parameters values: "
+        "<Command show-params> does not inherits from ExtraCommand.\n"
+    )
+
+
+def test_integrated_show_params_option(invoke, create_config):
+    @extra_command
     @option("--int-param1", type=int, default=10)
     @option("--int-param2", type=int, default=555)
     def show_params_cli(int_param1, int_param2):
@@ -559,87 +600,112 @@ def test_show_params_option(invoke, create_config):
     result = invoke(show_params_cli, *raw_args, color=False)
 
     assert result.exit_code == 0
+    assert f"debug: click_extra.raw_args: {raw_args!r}\n" in result.stderr
 
-    cli_config_option = [
-        p for p in show_params_cli.params if isinstance(p, ConfigOption)
-    ].pop()
     table = [
         (
-            "<ColorOption color>",
             "show-params-cli.color",
+            "click_extra.colorize.ColorOption",
+            "--color, --ansi / --no-color, --no-ansi",
             "bool",
+            "",
+            "✘",
             "SHOW_PARAMS_CLI_COLOR",
             True,
             True,
             "DEFAULT",
         ),
         (
-            "<ConfigOption config>",
             "show-params-cli.config",
+            "click_extra.config.ConfigOption",
+            "-C, --config CONFIG_PATH",
             "str",
+            "",
+            "✘",
             "SHOW_PARAMS_CLI_CONFIG",
             f"{Path(get_app_dir('show-params-cli')).resolve()}{sep}*.{{toml,yaml,yml,json,ini,xml}}",
             str(conf_path),
             "COMMANDLINE",
         ),
         (
-            "<HelpOption help>",
             "show-params-cli.help",
+            "click_extra.colorize.HelpOption",
+            "-h, --help",
             "bool",
+            "",
+            "✘",
             "SHOW_PARAMS_CLI_HELP",
             False,
             True,
             "COMMANDLINE",
         ),
         (
-            "<Option int_param1>",
             "show-params-cli.int_param1",
+            "cloup._params.Option",
+            "--int-param1 INTEGER",
             "int",
+            "",
+            "✓",
             "SHOW_PARAMS_CLI_INT_PARAM1",
             3,
             9999,
             "COMMANDLINE",
         ),
         (
-            "<Option int_param2>",
             "show-params-cli.int_param2",
+            "cloup._params.Option",
+            "--int-param2 INTEGER",
             "int",
+            "",
+            "✓",
             "SHOW_PARAMS_CLI_INT_PARAM2",
             555,
             555,
             "DEFAULT",
         ),
         (
-            "<ShowParamsOption show_params>",
             "show-params-cli.show_params",
+            "click_extra.config.ShowParamsOption",
+            "--show-params",
             "bool",
+            "",
+            "✘",
             "SHOW_PARAMS_CLI_SHOW_PARAMS",
             False,
             True,
             "COMMANDLINE",
         ),
         (
-            "<TimerOption time>",
             "show-params-cli.time",
+            "click_extra.commands.TimerOption",
+            "--time / --no-time",
             "bool",
+            "",
+            "✘",
             "SHOW_PARAMS_CLI_TIME",
             False,
             False,
             "DEFAULT",
         ),
         (
-            "<VerbosityOption verbosity>",
             "show-params-cli.verbosity",
+            "click_extra.logging.VerbosityOption",
+            "-v, --verbosity LEVEL",
             "str",
+            "",
+            "✘",
             "SHOW_PARAMS_CLI_VERBOSITY",
             "INFO",
             "DeBuG",
             "COMMANDLINE",
         ),
         (
-            "<VersionOption version>",
             "show-params-cli.version",
+            "click_extra.version.VersionOption",
+            "--version",
             "bool",
+            "",
+            "✘",
             "SHOW_PARAMS_CLI_VERSION",
             False,
             False,
